/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.apache.isis.core.metamodel.specloader.validator;

import java.util.Collections;
import java.util.Iterator;
import java.util.Set;

import com.google.common.collect.Sets;

public final class ValidationFailures implements Iterable<String> {

    private final Set<String> messages = Sets.newLinkedHashSet();
    
    public void add(String pattern, Object... arguments) {
        final String message = String.format(pattern, arguments);
        messages.add(message);
    }

    public void assertNone() {
        if (!occurred()) {
            return;
        }
        throw new MetaModelInvalidException(getMessagesBuf());
    }

    public boolean occurred() {
        return !messages.isEmpty();
    }

    public Set<String> getMessages() {
        return Collections.unmodifiableSet(messages);
    }

    private String getMessagesBuf() {
        final StringBuilder buf = new StringBuilder();
        int i=0;
        for (String message : messages) {
            buf.append(++i).append(": ").append(message).append("\n");
        }
        return buf.toString();
    }

    public int getNumberOfMessages() {
        return messages.size();
    }

    @Override
    public Iterator<String> iterator() {
        return getMessages().iterator();
    }


}
